@@ -27,5 +27,19 @@ extern volatile int burn_builtin_triggered_action;    /*  burn_is_aborting() */
 */
 int burn_init_catch_on_abort(int flag);
 
+/* ts B10606 */
+void *burn_alloc_mem(size_t size, size_t count, int flag);
+
+#define BURN_ALLOC_MEM(pt, typ, count) { \
+	pt= (typ *) burn_alloc_mem(sizeof(typ), (size_t) (count), 0); \
+	if(pt == NULL) { \
+		ret= -1; goto ex; \
+	} }
+
+#define BURN_FREE_MEM(pt) { \
+	if(pt != NULL) \
+		free((char *) pt); \
+	}
+
 
 #endif /* BURN__INIT_H */
