@@ -113,6 +113,10 @@ struct isoburn {
  */
  enum burn_disc_status fabricated_disc_status;
 
+ /* To be set if read errors occured during media evaluation.
+ */
+ int media_read_error;
+
  /* Eventual emulated table of content read from the chain of ISO headers
     on overwriteable media.
  */ 
