@@ -49,9 +49,11 @@ int Xorriso_auto_driveadr(struct XorrisO *xorriso, char *adr, char *result,
                           int flag)
 {
  int ret, is_known_mmc= 0;
- char *path_pt, libburn_adr[BURN_DRIVE_ADR_LEN + SfileadrL];
- char *abs_pt, abs_adr[SfileadrL];
+ char *path_pt, *libburn_adr= NULL;
+ char *abs_pt, *abs_adr= NULL;
 
+ Xorriso_alloc_meM(libburn_adr, char, BURN_DRIVE_ADR_LEN + SfileadrL);
+ Xorriso_alloc_meM(abs_adr, char, SfileadrL);
  path_pt= adr;
  if(strncmp(adr, "stdio:", 6) == 0)
    path_pt= adr + 6;
@@ -67,11 +69,11 @@ int Xorriso_auto_driveadr(struct XorrisO *xorriso, char *adr, char *result,
      Xorriso_msgs_submit(xorriso, 0,
               "Relative drive path given. Cannot determine working directory.",
                errno, "FAILURE", 0);
-     return(-1);
+     {ret= -1; goto ex;}
    }
    ret= Sfile_add_to_path(abs_adr, path_pt, 0);
    if(ret <= 0)
-     return(-1);
+     {ret= -1; goto ex;}
  }
 
  is_known_mmc= burn_drive_convert_fs_adr(path_pt, libburn_adr);
@@ -82,7 +84,7 @@ int Xorriso_auto_driveadr(struct XorrisO *xorriso, char *adr, char *result,
    goto ok;
  ret= Xorriso_is_in_patternlist(xorriso, xorriso->drive_blacklist, path_pt, 0);
  if(ret < 0)
-   return(ret);
+   goto ex;
  if(ret) {
    strcpy(xorriso->info_text, "Drive address ");
    Text_shellsafe(adr, xorriso->info_text, 1);
@@ -92,15 +94,15 @@ int Xorriso_auto_driveadr(struct XorrisO *xorriso, char *adr, char *result,
                                       ret - 1, 0),
                   xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(0);
+   {ret= 0; goto ex;}
  }
  /* if in greylist and not MMC and not stdio prefix: reject */
  if(is_known_mmc < 0)
-   return(ret);
+   goto ex;
  if(adr == path_pt && !is_known_mmc) { /* no prefix, no MMC */
    ret= Xorriso_is_in_patternlist(xorriso, xorriso->drive_greylist, path_pt,0);
    if(ret < 0)
-     return(ret);
+     goto ex;
    if(ret) {
      strcpy(xorriso->info_text, "Drive address ");
      Text_shellsafe(adr, xorriso->info_text, 1);
@@ -113,30 +115,34 @@ int Xorriso_auto_driveadr(struct XorrisO *xorriso, char *adr, char *result,
      Xorriso_msgs_submit(xorriso, 0,
                    "If the address is a legitimate target, prepend \"stdio:\"",
                    0, "HINT", 0);
-     return(0);
+     {ret= 0; goto ex;}
    }
  }
 ok:;
  if(strncmp(adr, "mmc:", 4) == 0) {
    if(Sfile_str(result, path_pt, 0) <= 0)
-     return(0);
+     {ret= 0; goto ex;}
  } else if(adr == path_pt && is_known_mmc <= 0) {
    Sfile_str(result, "stdio:", 0);
    if(Sfile_str(result, adr, 1) <= 0)
-     return(0);
+     {ret= 0; goto ex;}
  } else {
    if(Sfile_str(result, adr, 0) <= 0)
-     return(0);
+     {ret= 0; goto ex;}
  }
  if(strncmp(result, "stdio:", 6)==0) {
    if(xorriso->ban_stdio_write) {
      strcpy(xorriso->info_text, "Drive address banned by -ban_stdio_write : ");
      Text_shellsafe(result, xorriso->info_text, 1);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-     return(0);
+     {ret= 0; goto ex;}
    }
  }
- return(1);
+ ret= 1;
+ex:;
+ Xorriso_free_meM(libburn_adr);
+ Xorriso_free_meM(abs_adr);
+ return(ret);
 }
 
 
@@ -162,27 +168,30 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, int flag)
  IsoImage *volset = NULL;
  IsoNode *root_node;
  struct isoburn_read_opts *ropts= NULL;
- char libburn_adr[SfileadrL], *boot_fate, *sev;
+ char *libburn_adr= NULL, *boot_fate, *sev;
  size_t value_length;
  char *value= NULL;
  double num;
- char volid[33], adr_data[163], *adr_pt;
+ char volid[33], *adr_data= NULL, *adr_pt;
+
+ Xorriso_alloc_meM(libburn_adr, char, SfileadrL);
+ Xorriso_alloc_meM(adr_data, char, 163);
 
  if((flag&3)==0) { 
    sprintf(xorriso->info_text, 
          "XORRISOBURN program error : Xorriso_aquire_drive bit0+bit1 not set");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FATAL", 0);
-   return(-1);
+   {ret= -1; goto ex;}
  }
  ret= Xorriso_give_up_drive(xorriso, (flag&3)|8);
  if(ret<=0)
-   return(ret);
+   goto ex;
  if(flag & 1) 
    xorriso->isofs_st_out= time(0) - 1;
 
  ret= Xorriso_auto_driveadr(xorriso, adr, libburn_adr, 0);
  if(ret <= 0)
-   return(ret);
+   goto ex;
  if(strcmp(libburn_adr,"stdio:/dev/fd/1")==0) {
    if(xorriso->dev_fd_1<0) {
      sprintf(xorriso->info_text,
@@ -420,7 +429,7 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, int flag)
    if(xorriso->img_read_error_mode==2)
      sev= "FATAL";
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, sev, 0);
-   if(read_ret == ISO_SB_TREE_CORRUPTED && (xorriso->do_md5 & 1)) {
+   if(read_ret == (int) ISO_SB_TREE_CORRUPTED && (xorriso->do_md5 & 1)) {
      Xorriso_msgs_submit(xorriso, 0,
 "This might be false MD5 alarm if an add-on session was written by growisofs.",
         0, "HINT", 0);
@@ -529,6 +538,8 @@ ex:
  }
  if(ropts!=NULL)
    isoburn_ropt_destroy(&ropts, 0);
+ Xorriso_free_meM(libburn_adr);
+ Xorriso_free_meM(adr_data);
  return(ret);
 }
 
@@ -543,13 +554,12 @@ int Xorriso_give_up_drive(struct XorrisO *xorriso, int flag)
  int in_is_out_too, ret, do_eject;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
- char sfe[5*SfileadrL];
  
  in_is_out_too= (xorriso->in_drive_handle == xorriso->out_drive_handle);
  if((flag&4) && in_is_out_too && (flag&(1|2))) {
    if((flag&3)!=3) {
-     sprintf(xorriso->info_text,"Giving up for -eject whole -dev %s",
-               Text_shellsafe(xorriso->indev, sfe, 0));
+     sprintf(xorriso->info_text,"Giving up for -eject whole -dev ");
+     Text_shellsafe(xorriso->indev, xorriso->info_text, 1);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
    }
    flag|= 3; /* give up in/out drive to eject it */
@@ -681,28 +691,32 @@ ex:;
 int Xorriso_reaquire_outdev(struct XorrisO *xorriso, int flag)
 {
  int ret, aq_flag;
- char drive_name[SfileadrL], sfe[5*SfileadrL];
+ char *drive_name= NULL;
 
+ Xorriso_alloc_meM(drive_name, char, SfileadrL);
  aq_flag= flag&3;
  strcpy(drive_name, xorriso->outdev);
  Xorriso_give_up_drive(xorriso, aq_flag);
  if(aq_flag==0) { 
-   sprintf(xorriso->info_text,"Gave up -outdev %s",
-           Text_shellsafe(xorriso->outdev, sfe, 0));
+   sprintf(xorriso->info_text,"Gave up -outdev ");
+   Text_shellsafe(xorriso->outdev, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
-   return(1);
+   {ret= 1; goto ex;}
  }
- sprintf(xorriso->info_text,"Re-aquiring -outdev %s",
-         Text_shellsafe(drive_name, sfe, 0));
+ sprintf(xorriso->info_text,"Re-aquiring -outdev ");
+ Text_shellsafe(drive_name, xorriso->info_text, 1);
  Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
  ret= Xorriso_aquire_drive(xorriso, drive_name, aq_flag); 
  if(ret<=0) {
-   sprintf(xorriso->info_text,"Could not re-aquire -outdev %s",
-           Text_shellsafe(xorriso->outdev, sfe, 0));
+   sprintf(xorriso->info_text,"Could not re-aquire -outdev ");
+   Text_shellsafe(xorriso->outdev, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(ret);
+   goto ex;
  }
- return(1);
+ ret= 1;
+ex:;
+ Xorriso_free_meM(drive_name);
+ return(ret);
 }
 
 
@@ -790,6 +804,7 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
  char profile_name[80],*respt,*devadr, *typetext= "";
  struct burn_toc_entry toc_entry;
  struct burn_drive_info *dinfo;
+ struct burn_multi_caps *caps= NULL;
  struct burn_drive *drive;
  enum burn_disc_status s;
  char mem_text[80];
@@ -866,15 +881,32 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
    if(not_reconizable)
      sprintf(respt+strlen(respt), "is not recognizable\n");
    else
-     sprintf(respt+strlen(respt), "is written , is closed\n");
+     sprintf(respt+strlen(respt), "is written , is closed");
  } else if (s == BURN_DISC_APPENDABLE) {
-   sprintf(respt+strlen(respt), "is written , is appendable\n");
+   sprintf(respt+strlen(respt), "is written , is appendable");
  } else if (s == BURN_DISC_BLANK) {
-   sprintf(respt+strlen(respt), "is blank\n");
+   sprintf(respt+strlen(respt), "is blank");
  } else if (s == BURN_DISC_EMPTY)
-   sprintf(respt+strlen(respt), "is not present\n");
+   sprintf(respt+strlen(respt), "is not present");
  else
-   sprintf(respt+strlen(respt), "is not recognizable\n");
+   sprintf(respt+strlen(respt), "is not recognizable");
+ if(s == BURN_DISC_APPENDABLE || s == BURN_DISC_BLANK) {
+   ret= burn_disc_next_track_is_damaged(drive, 0);
+   if(ret & 1)
+     sprintf(respt+strlen(respt), " , but next track is damaged");
+   else if(ret & 2)
+     sprintf(respt+strlen(respt), " , but no writable address");
+   else if(profile_no == 0x14) { /* DVD-RW sequential */
+     ret= burn_disc_get_multi_caps(drive, BURN_WRITE_TAO, &caps, 0);
+     if(ret == 0)
+       sprintf(respt+strlen(respt), " , but will need -close on");
+     if(caps != NULL)
+       burn_disc_free_multi_caps(&caps);
+   } else if(profile_no == 0x15) { /* DVD-RW DL */
+     sprintf(respt+strlen(respt), " , but will need -close \"on\"");
+   }
+ }
+ strcat(respt, "\n");
  Xorriso_toc_line(xorriso, flag & 8);
 
  if(s == BURN_DISC_BLANK) {
@@ -1066,13 +1098,15 @@ ex:;
 
 int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
 {
- char adr[BURN_DRIVE_ADR_LEN];
- int i, j, max_dev_len= 1, pad;
+ char *adr= NULL;
+ int i, j, max_dev_len= 1, pad, ret;
  struct burn_drive_info *drive_list= NULL;
  unsigned int drive_count;
  char *respt, perms[8];
  struct stat stbuf;
 
+ Xorriso_alloc_meM(adr, char, BURN_DRIVE_ADR_LEN);
+
  sprintf(xorriso->info_text, "Beginning to scan for devices ...\n");
  Xorriso_info(xorriso,0);
 
@@ -1089,7 +1123,7 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
 
    sprintf(xorriso->info_text, "No drives found");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
-   return(0);
+   {ret= 0; goto ex;}
  }
  sprintf(xorriso->info_text, "Full drive scan done\n");
  Xorriso_info(xorriso,0);
@@ -1097,14 +1131,14 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
  sprintf(xorriso->info_text, "-----------------------------------------------------------------------------\n");
  Xorriso_info(xorriso,0);
  respt= xorriso->result_line;
- for(i= 0; i < drive_count && !(xorriso->request_to_abort); i++) {
+ for(i= 0; i < (int) drive_count && !(xorriso->request_to_abort); i++) {
    if(burn_drive_get_adr(&(drive_list[i]), adr)<=0)
      strcpy(adr, "-get_adr_failed-");
    Xorriso_process_msg_queues(xorriso,0);
-   if(strlen(adr)>max_dev_len)
+   if((int) strlen(adr) > max_dev_len)
      max_dev_len= strlen(adr);
  }
- for(i= 0; i < drive_count && !(xorriso->request_to_abort); i++) {
+ for(i= 0; i < (int) drive_count && !(xorriso->request_to_abort); i++) {
    if(burn_drive_get_adr(&(drive_list[i]), adr)<=0)
      strcpy(adr, "-get_adr_failed-");
    Xorriso_process_msg_queues(xorriso,0);
@@ -1133,7 +1167,10 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
 
  burn_drive_info_free(drive_list);
  Xorriso_process_msg_queues(xorriso,0);
- return(1);
+ ret= 1;
+ex:;
+ Xorriso_free_meM(adr);
+ return(ret);
 }
 
 
@@ -1566,13 +1603,16 @@ int Xorriso_check_md5_range(struct XorrisO *xorriso, off_t start_lba,
  struct burn_drive_info *dinfo= NULL;
  struct burn_drive *drive= NULL;
  off_t pos, data_count, to_read;
- char data[64 * 1024], data_md5[16];
+ char *data= NULL, data_md5[16];
  void *ctx = NULL;
 
  ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
                                 "on attempt to check session MD5 checksum", 0);
  if(ret <= 0)
    goto ex;
+
+ Xorriso_alloc_meM(data, char, 64 * 1024);
+
  ret= iso_md5_start(&ctx);
  if(ret <= 0) {
    Xorriso_no_malloc_memory(xorriso, NULL, 0);
@@ -1600,6 +1640,7 @@ ex:;
  Xorriso_process_msg_queues(xorriso,0);
  if(ctx != NULL)
    iso_md5_end(&ctx, data_md5);
+ Xorriso_free_meM(data);
  return(ret);
 }
 
@@ -1681,7 +1722,7 @@ int Xorriso_check_interval(struct XorrisO *xorriso, struct SpotlisT *spotlist,
  int start_lba= 0;
  struct burn_drive *drive;
  struct burn_drive_info *dinfo;
- char data[64*1024], sfe[5*SfileadrL];
+ char *data= NULL;
  off_t data_count, to_read, read_count= 0, write_amount;
  double pre_read_time, post_read_time, time_diff, total_time_diff= 0;
  double last_abort_file_time= 0;
@@ -1692,6 +1733,8 @@ int Xorriso_check_interval(struct XorrisO *xorriso, struct SpotlisT *spotlist,
  int tag_type= 0, valid, was_sb_tag= 0, in_track_gap= 0;
  char *comparison= "", *sev_text= "DEBUG", *tag_type_name= "";
 
+ Xorriso_alloc_meM(data, char, 64*1024);
+
  ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
                                 "on attempt to check media readability",
                                 2 * !!job->use_dev);
@@ -1840,21 +1883,23 @@ abort_check:;
      if(ctx != NULL) {
        for(j= 0; j < to_read; j++) {
          lba=  i + j + from_lba;
-         if(lba < md5_start)
+         if(lba < (uint32_t) md5_start)
        continue;
          ret= 0;
-         if(lba > md5_start + 16 &&
+         if(lba > (uint32_t) (md5_start) + 16 &&
             (next_tag == 0 || chain_broken || lba == next_tag)) {
            ret= iso_util_decode_md5_tag(data + j * 2048, &tag_type,
                                         &pos, &range_start, &range_size,
                                         &next_tag, tag_md5, !!chain_broken);
          }
-         valid= (ret == 1 || ret == ISO_MD5_AREA_CORRUPTED) && pos == lba;
-         if(valid && tag_type == 2 && (lba < md5_start + 32 || in_track_gap)) {
+         valid= (ret == 1 || ret == (int) ISO_MD5_AREA_CORRUPTED) &&
+                pos == lba;
+         if(valid && tag_type == 2 && (lba < ((uint32_t) md5_start) + 32 ||
+            in_track_gap)) {
            tag_type_name= "superblock";
            was_sb_tag= 1;
-           if(in_track_gap && range_start != md5_start && range_start < lba &&
-              lba - range_start <= j) {
+           if(in_track_gap && range_start != ((uint32_t) md5_start) &&
+              range_start < lba && lba - range_start <= (uint32_t) j) {
              /* Looking for next session : start computing in hindsight.
                 Session start and superblock tag are supposed to be in the
                 same 64 kB chunk.
@@ -1883,7 +1928,7 @@ abort_check:;
            tag_type_name= "";
          }
          if (tag_type_name[0]) {
-           if(range_start != md5_start) {
+           if(range_start != (uint32_t) md5_start) {
              sprintf(xorriso->info_text,
                  "Found MD5 %s tag which covers different data range",
                  tag_type_name);
@@ -1902,7 +1947,7 @@ abort_check:;
              }
              iso_md5_end(&cloned_ctx, md5);
 
-             if(ret == ISO_MD5_AREA_CORRUPTED) {
+             if(ret == (int) ISO_MD5_AREA_CORRUPTED) {
                comparison= "CORRUPTED";
                sev_text= "WARNING";
                md5_spot_value= Xorriso_read_quality_md5_mismatcH;
@@ -1966,9 +2011,9 @@ abort_check:;
                  SEEK_SET);
          if(ret == -1) {
 failed_to_write:;
-           sprintf(xorriso->info_text, "Cannot write %d bytes to lba %d of %s",
-                   (int) data_count, i + from_lba,
-                   Text_shellsafe(job->data_to_path, sfe, 0));
+           sprintf(xorriso->info_text, "Cannot write %d bytes to lba %d of ",
+                   (int) data_count, i + from_lba);
+           Text_shellsafe(job->data_to_path, xorriso->info_text, 1);
            Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno,
                                "FAILURE", 0);
            {ret= 0; goto ex;}
@@ -2034,6 +2079,7 @@ failed_to_write:;
 ex:
  if(ctx != NULL)
    iso_md5_end(&ctx, md5);
+ Xorriso_free_meM(data);
  return(ret);
 }
 
@@ -2260,6 +2306,8 @@ ex:;
    Spotlist_destroy(spotlist, 0);
  if(caps!=NULL)
    burn_disc_free_multi_caps(&caps);
+ if(isoburn_disc != NULL)
+   isoburn_toc_disc_free(isoburn_disc);
  return(ret);
 }
 
