@@ -212,7 +212,7 @@ int Decode_date_input_format(struct tm *erg, char *text, int flag)
 
  current_time= time(0);
  now= localtime(&current_time);
- for(i=0;i<sizeof(struct tm);i++)
+ for(i= 0; i < (int) sizeof(struct tm); i++)
    ((char *) erg)[i]= ((char *) now)[i];
 
  l= strlen(text);
@@ -890,7 +890,7 @@ int Sregex_resolve_var(char *form, char *vars[][2], int num_vars,
    /* look for start mark */
    spt= strstr(rpt, start);
    if(spt == NULL) {
-     if((wpt - result) + strlen(rpt) >= result_size)
+     if((wpt - result) + (int) strlen(rpt) >= result_size)
        return(0);
      strcpy(wpt, rpt);
      wpt+= strlen(wpt);
@@ -948,7 +948,7 @@ int Sregex_resolve_var(char *form, char *vars[][2], int num_vars,
    if(i < num_vars) {
      /* substitute found variable */
      l_v= strlen(vars[i][0]);
-     if((wpt - result) + strlen(vars[i][1]) >= result_size)
+     if((wpt - result) + (int) strlen(vars[i][1]) >= result_size)
        return(0);
      strcpy(wpt, vars[i][1]);
      rpt= npt + strlen(vars[i][0]) + l_e;
@@ -980,23 +980,31 @@ int Sregex_resolve_var(char *form, char *vars[][2], int num_vars,
 int Sregex_match(char *pattern, char *text, int flag)
 {
  int ret;
- char re_text[2*SfileadrL];
+ char *re_text= NULL;
  regex_t re;
  regmatch_t match[1];
 
+ re_text= TSOB_FELD(char, 2 * SfileadrL);
+ if(re_text == NULL)
+   {ret= -1; goto ex;}
+
  Xorriso__bourne_to_reg(pattern, re_text, 0);
  ret= regcomp(&re, re_text, 0);
  if(ret != 0) 
-   return(-1);
+   {ret= -1; goto ex;}
  if(flag & 1) {
    regfree(&re);
-   return(1);
+   {ret= 1; goto ex;}
  }
  ret= regexec(&re, text, 1, match, 0);
  regfree(&re);
  if(ret != 0)
-     return(0);
- return(1);
+   goto ex;
+ ret= 1;
+ex:;
+ if(re_text != NULL)
+   free(re_text);
+ return(ret);
 }
 
 
@@ -1005,19 +1013,22 @@ int Sregex_match(char *pattern, char *text, int flag)
 
 
 /* @param flag bit0= append to out_text rather than overwrite it
+               bit1= length limit is 10 * SfileadrL rather than 5 *
 */
 char *Text_shellsafe(char *in_text, char *out_text, int flag)
 {
- int l,i,ol= 0,w=0;
+ int l,i,w=0, limit= 5 * SfileadrL;
 
  if(flag&1)
-   ol= w= strlen(out_text);
+   w= strlen(out_text);
+ if(flag & 2)
+   limit= 10 * SfileadrL;
  /* enclose everything by hard quotes */
  l= strlen(in_text);
  out_text[w++]= '\'';
  for(i=0;i<l;i++){
    if(in_text[i]=='\''){
-     if(w+7>5*SfileadrL+ol)
+     if(w + 7 > limit)
        goto overflow;
      /* escape hard quote within the text */
      out_text[w++]= '\'';
@@ -1026,7 +1037,7 @@ char *Text_shellsafe(char *in_text, char *out_text, int flag)
      out_text[w++]= '"';
      out_text[w++]= '\'';
    } else {
-     if(w+3>5*SfileadrL) {
+     if(w + 3 > limit) {
 overflow:;
        strncpy(out_text, "'xorriso: TEXT MUCH TOO LONG ...   ",33);
  break;
