@@ -282,7 +282,6 @@ int Xorriso_cpmv_args(struct XorrisO *xorriso, char *cmd,
 {
  int destc= 0, is_dir=0, end_idx, ret, i;
  char **destv= NULL;
- char sfe[5*SfileadrL];
 
  end_idx= Xorriso_end_idx(xorriso, argc, argv, *idx,
                           (xorriso->do_iso_rr_pattern==1)|(flag&2));
@@ -318,8 +317,9 @@ int Xorriso_cpmv_args(struct XorrisO *xorriso, char *cmd,
      for(i= 0; i<*optc; i++)
        Xorriso_msgs_submit(xorriso, 0, (*optv)[i], 0, "ERRFILE", 0);
    sprintf(xorriso->info_text,
-        "%s: more than one origin given, destination is a non-directory: %s",
-        cmd, Text_shellsafe(destv[0], sfe, 0));
+        "%s: more than one origin given, destination is a non-directory: ",
+        cmd);
+   Text_shellsafe(destv[0], xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    {ret= 0; goto ex;}
  }
@@ -390,7 +390,7 @@ int Xorriso_decode_load_adr(struct XorrisO *xorriso, char *cmd,
 int Xorriso_check_name_len(struct XorrisO *xorriso, char *name, int size,
                            char *cmd, int flag)
 {
- if(strlen(name) >= size) {
+ if((int) strlen(name) >= size) {
    sprintf(xorriso->info_text,
            "Name too long with option %s (%d > %d)", cmd,
            (int) strlen(name), size - 1);
@@ -411,7 +411,7 @@ int Xorriso_normalize_command(struct XorrisO *xorriso, char *original_cmd,
  int was_dashed= 0;
  char *dash_pt;
 
- if(strlen(original_cmd)>= sizeof_cmd_data) {
+ if((int) strlen(original_cmd) >= sizeof_cmd_data) {
    if(argno>=0)
      sprintf(xorriso->info_text, "Oversized argument #%d (length %d)\n",
              argno, (int) strlen(original_cmd));
@@ -444,8 +444,8 @@ int Xorriso_normalize_command(struct XorrisO *xorriso, char *original_cmd,
 int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
                        int *count, int flag)
 {
- int ret, was_dashed= 0, i;
- char *cmd, cmd_data[2*SfileadrL];
+ int ret, was_dashed= 0, i, cmd_data_size= 2 * SfileadrL;
+ char *cmd, *cmd_data= NULL;
  static char arg0_commands[][40]= {
     "ban_stdio_write","close_filter_list","commit","devices","end",
     "for_backup", "help",
@@ -458,7 +458,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "abort_on","acl","add_plainly","application_id","auto_charset",
     "abstract_file",
     "backslash_codes","blank","biblio_file",
-    "calm_drive","cd","cdi","cdx","charset","close",
+    "calm_drive","cd","cdi","cdx","charset","close","close_damaged",
     "commit_eject","compliance","copyright_file",
     "dev","dialog","disk_dev_ino","disk_pattern","displacement",
     "dummy","dvd_obs","early_stdio_test", "eject",
@@ -471,7 +471,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "preparer_id","print","print_info","print_mark","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
     "reassure","report_about","rom_toc_scan","scsi_log",
-    "session_log","speed","split_size","status","status_history_max",
+    "session_log","signal_handling",
+    "speed","split_size","status","status_history_max",
     "stdio_sync","stream_recording","system_id","temp_mem_limit",
     "uid","unregister_filter","use_readline","volid","volset_id",
     "xattr","zisofs",
@@ -513,45 +514,47 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     ""
  };
 
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+
  *count= 0;
  if(argc<=0)
-   return(-1);
+   {ret= -1; goto ex;}
  ret= Xorriso_normalize_command(xorriso, argv[0], -1,
-                                cmd_data, sizeof(cmd_data), &cmd, 0);
+                                cmd_data, cmd_data_size, &cmd, 0);
  if(ret<0)
-   return(ret);
+   goto ex;
  was_dashed= (ret>0);
  if(cmd[0]=='#' || cmd[0]==0 || strcmp(cmd, xorriso->list_delimiter) == 0) {
    /* ignore: comment line , empty option , orphaned list delimiter */
-   return(1);
+   {ret= 1; goto ex;}
  }
  for(i=0; arg0_commands[i][0]!=0; i++)
    if(strcmp(arg0_commands[i], cmd)==0)
-     return(1);
+     {ret= 1; goto ex;}
  *count= 1;
  for(i=0; arg1_commands[i][0]!=0; i++)
    if(strcmp(arg1_commands[i], cmd)==0)
-     return(1);
+     {ret= 1; goto ex;}
  *count= 2;
  for(i=0; arg2_commands[i][0]!=0; i++)
    if(strcmp(arg2_commands[i], cmd)==0)
-     return(1);
+     {ret= 1; goto ex;}
  *count= 3;
  for(i=0; arg3_commands[i][0]!=0; i++)
    if(strcmp(arg3_commands[i], cmd)==0)
-     return(1);
+     {ret= 1; goto ex;}
  *count= 4;
  for(i=0; arg4_commands[i][0]!=0; i++)
    if(strcmp(arg4_commands[i], cmd)==0)
-     return(1);
+     {ret= 1; goto ex;}
  *count= 0;
  for(i=0; argn_commands[i][0]!=0; i++)
    if(strcmp(argn_commands[i], cmd)==0) {
      ret= Xorriso_end_idx(xorriso, argc, argv, 1, 1);
      if(ret<1)
-       return(ret);
+       goto ex;
      *count= ret-1;
-     return(1);
+     {ret= 1; goto ex;}
    }
 
  if(!(flag&1)) {
@@ -559,7 +562,10 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
  }
 
- return(2+!was_dashed);
+ ret= 2 + !was_dashed;
+ex:
+ Xorriso_free_meM(cmd_data);
+ return(ret);
 }
 
 
@@ -574,11 +580,14 @@ return:
  <=0 error , 1 = success , 2 = problem event ignored , 3 = end program run
 */
 {
- int ret, was_dashed, end_ret, num1, num2;
- char *cmd, *original_cmd, cmd_data[2*SfileadrL], *arg1, *arg2;
+ int ret, was_dashed, end_ret, num1, num2, cmd_data_size= 2 * SfileadrL;
+ int mem_idx;
+ char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
  
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+
  if(xorriso==NULL)
-   return(0);
+   {ret= 0; goto ex;}
  if(xorriso->is_dialog) {
    xorriso->result_line_counter= xorriso->result_page_counter= 0;
    if(xorriso->result_page_length<0)
@@ -615,7 +624,7 @@ next_command:;
  was_dashed= 0;
 
  ret= Xorriso_normalize_command(xorriso, original_cmd, -1,
-                                cmd_data, sizeof(cmd_data), &cmd, 0);
+                                cmd_data, cmd_data_size, &cmd, 0);
  if(ret<0)
    goto eval_any_problems;
  was_dashed= ret;
@@ -757,6 +766,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_close(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"close_damaged")==0) {
+   (*idx)++;
+   ret= Xorriso_option_close_damaged(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"close_filter_list")==0) {
    ret= Xorriso_option_close_filter_list(xorriso, 0);
 
@@ -865,7 +878,7 @@ next_command:;
    end_ret= Xorriso_option_end(xorriso, 0);
    ret= Xorriso_eval_problem_status(xorriso, ret, 0);
    if(ret<0)
-     return(ret);
+     goto ex;
    if(end_ret!=2)
      {ret= 3; goto ex;}
 
@@ -1230,7 +1243,7 @@ next_command:;
    end_ret= Xorriso_option_end(xorriso, 1);
    ret= Xorriso_eval_problem_status(xorriso, ret, 0);
    if(ret<0)
-     return(ret);
+     goto ex;
    if(end_ret!=2)
      {ret= 3; goto ex;}
 
@@ -1295,6 +1308,10 @@ next_command:;
    ret= Xorriso_option_set_filter(xorriso, "", argc, argv, idx,
                                   (strcmp(cmd,"show_stream_r")==0) | 2 | 4);
 
+ } else if(strcmp(cmd,"signal_handling")==0) {
+   (*idx)++;
+   ret= Xorriso_option_signal_handling(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"speed")==0) {
    (*idx)++;
    ret= Xorriso_option_speed(xorriso, arg1, 0);
@@ -1399,22 +1416,24 @@ unknown_option:;
    if(xorriso->add_plainly<=0)
      goto unknown_option;
 add_plain_argument:;
+   mem_idx= *idx;
    (*idx)--;
    ret= Xorriso_option_add(xorriso, (*idx)+1, argv, idx, 0);
-   (*idx)++;
+   (*idx)= mem_idx;
 
  }
 
 eval_any_problems:
  ret= Xorriso_eval_problem_status(xorriso, ret, 0);
  if(ret<0)
-   return(ret);
+   goto ex;
 
  if(*idx<argc)
    goto next_command;
 
 ex:;
  fflush(stdout);
+ Xorriso_free_meM(cmd_data);
  return(ret);
 }
 
@@ -1487,11 +1506,13 @@ ex:;
 
 int Xorriso_dialog(struct XorrisO *xorriso, int flag)
 {
- int ret,first_round;
- char line[2*SfileadrL];
+ int ret,first_round, line_size= 2 * SfileadrL;
+ char *line= NULL;
+
+ Xorriso_alloc_meM(line, char, line_size);
 
  if(!xorriso->dialog)
-   return(1);
+   {ret= 1; goto ex;}
  if(xorriso->abort_on_is_default)
    Xorriso_option_abort_on(xorriso, "NEVER", 0);
  xorriso->is_dialog= 1;
@@ -1509,7 +1530,7 @@ int Xorriso_dialog(struct XorrisO *xorriso, int flag)
      sprintf(xorriso->info_text,"enter option and arguments :\n");
      Xorriso_info(xorriso,0);
      Xorriso_mark(xorriso,0);
-     ret= Xorriso_dialog_input(xorriso,line,sizeof(line),4);
+     ret= Xorriso_dialog_input(xorriso,line, line_size, 4);
      if(ret<=0)
  break;
    }
@@ -1528,6 +1549,7 @@ int Xorriso_dialog(struct XorrisO *xorriso, int flag)
  ret= 1;
 ex:;
  xorriso->is_dialog= 0;
+ Xorriso_free_meM(line);
  return(ret);
 }
 
@@ -1545,17 +1567,23 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
 */
 {
  int i, ret, was_dashed, num2, arg_count;
+ int advice, mem_add_plainly, error_seen= 0;
  int was_report_about= 0, was_abort_on= 0, was_return_with= 0;
- char *cmd, *original_cmd, cmd_data[5*SfileadrL], *arg1, *arg2;
+ int was_signal_handling= 0, was_scsi_log= 0, cmd_data_size= 5 * SfileadrL;
+ char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
  char mem_list_delimiter[81];
 
  strcpy(mem_list_delimiter, xorriso->list_delimiter);
+ mem_add_plainly= xorriso->add_plainly;
+
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+
  for(i=1+(flag&1);i<argc;i++) {
    original_cmd= cmd= argv[i];
    was_dashed= 0;
 
    was_dashed= Xorriso_normalize_command(xorriso, original_cmd, i,
-                                         cmd_data, sizeof(cmd_data), &cmd, 0);
+                                         cmd_data, cmd_data_size, &cmd, 0);
    if(was_dashed<0)
      {ret= -1; goto ex;}
 
@@ -1587,7 +1615,7 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
    } else if(i==1 && strcmp(cmd,"no_rc")==0) {
      ret= Xorriso_option_no_rc(xorriso, 0);
      if(ret<=0)
-       goto ex;
+       error_seen= 1;
    } else if(xorriso->argument_emulation >= 1 &&
              xorriso->argument_emulation <= 2) {
      {ret= 1; goto ex;}
@@ -1600,15 +1628,17 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
         "stdio:/dev/fd/1" is interpreted as drive address.
      */
 protect_stdout:;
-     Xorriso_protect_stdout(xorriso, 0);
-     sprintf(xorriso->info_text,
+     ret= Xorriso_protect_stdout(xorriso, 0);
+     if(ret == 1) {
+       sprintf(xorriso->info_text,
              "Encountered  -  or  stdio:/dev/fd/1  as possible write target.");
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
-     sprintf(xorriso->info_text,
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+       sprintf(xorriso->info_text,
              "Redirecting nearly all text message output to stderr.");
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
-     sprintf(xorriso->info_text, "Disabling use of libreadline.");
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+       sprintf(xorriso->info_text, "Disabling use of libreadline.");
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+     }
 
    } else if(strcmp(cmd,"abort_on")==0 && was_dashed == 1) {
      i++;
@@ -1631,30 +1661,65 @@ protect_stdout:;
      was_return_with= 1;
 
    } else if(strcmp(cmd,"as")==0 && was_dashed == 1) {
-     if(strcmp(arg1, "mkisofs")==0 || strcmp(arg1, "genisoimage")==0 ||
-        strcmp(arg1, "genisofs")==0 || strcmp(arg1, "xorrisofs")==0)
-       goto protect_stdout;
+     ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
+     if(ret == 1)
+       i+= arg_count;
+     if(was_dashed == 1)
+       if(strcmp(arg1, "mkisofs")==0 || strcmp(arg1, "genisoimage")==0 ||
+          strcmp(arg1, "genisofs")==0 || strcmp(arg1, "xorrisofs")==0)
+         goto protect_stdout;
 
    } else if(strcmp(cmd, "list_delimiter") == 0) {
      /* Needed for interpreting other args. Gets reset after prescan. */
      i++;
      ret= Xorriso_option_list_delimiter(xorriso, arg1, 0);
      if(ret <= 0)
-       goto ex;
+       error_seen= 1;
 
+   } else if(strcmp(cmd, "add_plainly") == 0) {
+     i++;
+     ret= Xorriso_option_add_plainly(xorriso, arg1, 0);
+     if(ret <= 0)
+       error_seen= 1;
+     if(xorriso->add_plainly == 3) {
+       /* All further arguments count as pathspecs */
+       {ret= 1; goto ex;}
+     }
    } else if(strcmp(cmd, "scsi_log") == 0 && was_dashed == 1) {
      i++;
-     Xorriso_option_scsi_log(xorriso, arg1, 0);
+     if(!was_scsi_log)
+        Xorriso_option_scsi_log(xorriso, arg1, 0);
+     was_scsi_log= 1;
+
+   } else if(strcmp(cmd, "signal_handling") == 0 && was_dashed == 1) {
+     i++;
+     if(!was_signal_handling)
+       Xorriso_option_signal_handling(xorriso, arg1, 1); /* no install */
+     was_signal_handling= 1;
 
    } else {
-     ret= Xorriso_count_args(xorriso, argc-i, argv+i, &arg_count, 1);
-     if(ret==1)
+     ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
+     if(ret == 1) {
        i+= arg_count;
+     } else if((flag & 2) && ((was_dashed && xorriso->add_plainly <= 1) ||
+                              xorriso->add_plainly <= 0)) {
+       sprintf(xorriso->info_text, "Not a known option:  '%s'\n",
+               original_cmd);
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+       error_seen= 1;
+     }
    }
  }
  ret= 1;
 ex:;
  strcpy(xorriso->list_delimiter, mem_list_delimiter);
+ xorriso->add_plainly= mem_add_plainly;
+ Xorriso_free_meM(cmd_data);
+ if(error_seen && ret > 0) {
+   advice= Xorriso_eval_problem_status(xorriso, 0, 0);
+   if(advice < 0)
+     ret= -1;
+ }
  return(ret);
 }
 
@@ -1663,7 +1728,9 @@ int Xorriso_read_as_mkisofsrc(struct XorrisO *xorriso, char *path, int flag)
 {
  int ret, linecount= 0;
  FILE *fp= NULL;
- char *sret, line[SfileadrL], *cpt, *wpt;
+ char *sret, *line= NULL, *cpt, *wpt;
+
+ Xorriso_alloc_meM(line, char, SfileadrL);
 
  ret= Xorriso_afile_fopen(xorriso, path, "rb", &fp, 1 | 2);
  if(ret <= 0)
@@ -1743,6 +1810,7 @@ int Xorriso_read_as_mkisofsrc(struct XorrisO *xorriso, char *path, int flag)
 ex:
  if(fp != NULL)
    fclose(fp);
+ Xorriso_free_meM(line);
  return(ret);
 }
 
@@ -1752,19 +1820,21 @@ ex:
  */
 int Xorriso_read_mkisofsrc(struct XorrisO *xorriso, int flag)
 {
- char path[SfileadrL], *cpt;
+ char *path= NULL, *cpt;
  int ret;
 
+ Xorriso_alloc_meM(path, char, SfileadrL);
+
  ret= Xorriso_read_as_mkisofsrc(xorriso, "./.mkisofsrc", 0);
  if(ret > 0)
-   return(ret);
+   goto ex;
  cpt= getenv("MKISOFSRC");
  if(cpt != NULL) {
    strncpy(path, cpt, SfileadrL - 1);
    path[SfileadrL - 1]= 0;
    ret= Xorriso_read_as_mkisofsrc(xorriso, path, 0);
    if(ret > 0)
-     return(ret);
+     goto ex;
  }
  cpt= getenv("HOME");
  if(cpt != NULL) {
@@ -1773,7 +1843,7 @@ int Xorriso_read_mkisofsrc(struct XorrisO *xorriso, int flag)
    strcat(path, "/.mkisofsrc");
    ret= Xorriso_read_as_mkisofsrc(xorriso, path, 0);
    if(ret > 0)
-     return(ret);
+     goto ex;
  }
  strcpy(path, xorriso->progname);
  cpt= strrchr(path, '/');
@@ -1781,10 +1851,13 @@ int Xorriso_read_mkisofsrc(struct XorrisO *xorriso, int flag)
    strcpy(cpt + 1, ".mkisofsrc");
    ret= Xorriso_read_as_mkisofsrc(xorriso, path, 0);
    if(ret > 0)
-     return(ret);
+     goto ex;
  }
  /* no .mkisofsrc file found */
- return(2); 
+ ret= 2;
+ex:;
+ Xorriso_free_meM(path);
+ return(ret);
 }
 
 
@@ -1858,9 +1931,11 @@ int Xorriso_program_arg_bsl(struct XorrisO *xorriso, int argc, char ***argv,
 int Xorriso_make_abs_adr(struct XorrisO *xorriso, char *wd, char *name,
                              char adr[], int flag)
 {
- char norm_adr[SfileadrL];
+ char *norm_adr= NULL;
  int ret;
 
+ Xorriso_alloc_meM(norm_adr, char, SfileadrL);
+
  if((wd[0]!=0 || (flag&4)) && !((flag&1) && name[0]=='/')) {
    if(strlen(wd)+1>=SfileadrL)
      goto much_too_long;
@@ -1869,7 +1944,7 @@ int Xorriso_make_abs_adr(struct XorrisO *xorriso, char *wd, char *name,
      if(Sfile_add_to_path(adr, name, 0)<=0) {
 much_too_long:;
        Xorriso_much_too_long(xorriso, (int) (strlen(adr)+strlen(name)+1), 2);
-       return(0);
+       {ret= 0; goto ex;}
      }
  } else {
    if(strlen(name)+1>=SfileadrL)
@@ -1880,12 +1955,15 @@ much_too_long:;
    ret= Xorriso_normalize_img_path(xorriso, "", adr, norm_adr,
                                    1|2|((flag&8)>>1));
    if(ret<=0)
-     return(ret);
+     goto ex;
    if(norm_adr[0]==0)
      strcpy(norm_adr, "/");
    strcpy(adr, norm_adr);
  }
- return(1);
+ ret= 1;
+ex:;
+ Xorriso_free_meM(norm_adr);
+ return(ret);
 }
 
 
@@ -1957,11 +2035,12 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
                   int *status, int flag)
 {
  int ret, argc= 0;
- char **argv= NULL, *pathlist= NULL, *cpt, *npt, prog[SfileadrL];
- char sfe[5*SfileadrL];
+ char **argv= NULL, *pathlist= NULL, *cpt, *npt, *prog= NULL;
  pid_t child_pid;
  struct stat stbuf;
 
+ Xorriso_alloc_meM(prog, char, 5 * SfileadrL);
+
  wait3(NULL,WNOHANG,NULL); /* just to remove any old dead child */
 
  ret= Sfile_make_argv("", cmd, &argc, &argv, 1|4|128);
@@ -1991,8 +2070,8 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
        prog[0]= 0;
      }
      if(prog[0] == 0) {
-       sprintf(xorriso->info_text, "Cannot find external program %s",
-               Text_shellsafe(argv[0], sfe, 0));
+       sprintf(xorriso->info_text, "Cannot find external program ");
+       Text_shellsafe(argv[0], xorriso->info_text, 1);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        ret= 0; goto ex;
      }
@@ -2005,10 +2084,12 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
  if(child_pid==0) {
                      /* this is the child process */
 
-   sprintf(xorriso->info_text, "Executing external program %s",
-                               Text_shellsafe(prog, sfe, 0));
+   sprintf(xorriso->info_text, "Executing external program ");
+   Text_shellsafe(prog, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
 
+   Xorriso_destroy(&xorriso, 0); /* reduce memory foot print */
+
    execv(prog, argv); /* should never come back */
    fprintf(stderr,"--- execution of shell command failed:\n");
    fprintf(stderr,"    %s\n",cmd);
@@ -2048,6 +2129,7 @@ int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
 ex:
  Sfile_make_argv("", "", &argc, &argv, 2);
  Sregex_string(&pathlist, NULL, 0);
+ Xorriso_free_meM(prog);
  return(ret);
 }
 
@@ -2057,7 +2139,6 @@ ex:
 int Xorriso_path_is_excluded(struct XorrisO *xorriso, char *path, int flag)
 {
  int ret;
- char sfe[5*SfileadrL];
 
  if(!(xorriso->disk_excl_mode&1)) /* exclusion is off */
    return(0);
@@ -2071,8 +2152,9 @@ int Xorriso_path_is_excluded(struct XorrisO *xorriso, char *path, int flag)
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FATAL", 0);
  }
  if(ret>0 && (flag&1)) {
-   sprintf(xorriso->info_text, "Disk path parameter excluded by %s : %s",
-          (ret==1 ? "-not_paths" : "-not_leaf"), Text_shellsafe(path, sfe, 0));
+   sprintf(xorriso->info_text, "Disk path parameter excluded by %s : ",
+          (ret==1 ? "-not_paths" : "-not_leaf"));
+   Text_shellsafe(path, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
  }
  return(ret);
