@@ -337,7 +337,7 @@ int Sfile_scale(double value, char *result, int siz, double thresh, int flag)
    avail_siz= siz-1;
    sprintf(form,"%%.f");
    sprintf(result,"%.f",value);
-   if(strlen(result)<=avail_siz-2) 
+   if(((int) strlen(result)) <= avail_siz - 2) 
      dec_siz= 1;                                  /* we are very modest */
  }
  if(scale_c=='b' && (flag&1)) {
@@ -548,7 +548,7 @@ int Sfile_bsl_encoder(char **result, char *text, size_t text_len, int flag)
  int count, sq_open= 0, dq_open= 0;
 
  count= 0;
- for(rpt= text; rpt - text < text_len; rpt++) {
+ for(rpt= text; (size_t) (rpt - text) < text_len; rpt++) {
    count++;
    if(flag & 8) {
       if(!(*rpt <= 42 || (*rpt >= 59 && *rpt <= 63) ||
@@ -564,7 +564,7 @@ int Sfile_bsl_encoder(char **result, char *text, size_t text_len, int flag)
  (*result)= wpt= calloc(count + 1, 1);
  if(wpt == NULL)
    return(-1);
- for(rpt= text; rpt - text < text_len; rpt++) {
+ for(rpt= text; (size_t) (rpt - text) < text_len; rpt++) {
    if(*rpt == '\'')
      sq_open= !(sq_open || dq_open);
    if(*rpt == '"')
@@ -827,7 +827,7 @@ int Sfile_home_adr_s(char *filename, char *fileadr, int fa_size, int flag)
  home= getenv("HOME");
  if(home==NULL)
    return(0);
- if(strlen(home)+strlen(filename)+1>=fa_size)
+ if((int) (strlen(home) + strlen(filename) + 1) >= fa_size)
    return(-1);
  strcpy(fileadr,home);
  if(filename[0]!=0){
@@ -857,7 +857,7 @@ int Sfile_decode_datestr(struct tm *reply, char *text, int flag)
 
  current_time= time(0);
  now= localtime(&current_time);
- for(i=0;i<sizeof(struct tm);i++)
+ for(i=0; i < (int) sizeof(struct tm); i++)
    ((char *) reply)[i]= ((char *) now)[i];
 
  if(text[0]<'0'|| (text[0]>'9' && text[0]<'A') || text[0]>'Z')
