@@ -58,6 +58,60 @@
 #include "write_run.h"
 
 
+/* @param flag bit0= talk of -as cdrecord -multi rather than of -close
+*/
+int Xorriso_check_multi(struct XorrisO *xorriso, struct burn_drive *drive,
+                        int flag)
+{
+ int profile_no= 0, ret;
+ struct burn_multi_caps *caps= NULL;
+ char profile_name[80];
+
+ if(!xorriso->do_close) {
+   burn_disc_get_profile(drive, &profile_no, profile_name);
+   if(profile_no == 0x14) { /* DVD-RW sequential */
+     ret= burn_disc_get_multi_caps(drive, BURN_WRITE_TAO, &caps, 0);
+     if(caps != NULL)
+       burn_disc_free_multi_caps(&caps);
+     if(ret == 0) {
+       if(flag & 1) {
+         sprintf(xorriso->info_text,
+                "This DVD-RW media can only be written without option -multi");
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+         sprintf(xorriso->info_text,
+                 "Possibly it was blanked by blank=deformat_quickest");
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "HINT", 0);
+         sprintf(xorriso->info_text,
+                 "After writing a session without -multi, apply blank=all");
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "HINT", 0);
+       } else {
+         sprintf(xorriso->info_text,
+                 "This DVD-RW media can only be written with -close \"on\"");
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+         sprintf(xorriso->info_text,
+                 "Possibly it was blanked by -blank \"deformat_quickest\"");
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "HINT", 0);
+         sprintf(xorriso->info_text,
+          "After writing a session with -close \"on\", apply -blank \"all\"");
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "HINT", 0);
+       }
+       return(0);
+     }
+   } else if(profile_no == 0x15) { /* DVD-RW DL */
+     if(flag & 1)
+       sprintf(xorriso->info_text,
+               "DVD-R DL media can only be written without option -multi");
+     else
+       sprintf(xorriso->info_text,
+               "DVD-R DL media can only be written with -close \"on\"");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+ }
+ return(1);
+}
+
+
 int Xorriso_make_write_options(
         struct XorrisO *xorriso, struct burn_drive *drive,
         struct burn_write_opts **burn_options, int flag)
@@ -252,7 +306,7 @@ int Xorriso_set_system_area(struct XorrisO *xorriso, struct burn_drive *drive,
  int ret, options, system_area_options, iso_lba= -1, start_lba, image_blocks;
  char volid[33];
  FILE *fp= NULL;
- char buf[32768], *bufpt= NULL;
+ char *buf= NULL, *bufpt= NULL;
  off_t hd_lba;
  unsigned char *ub;
  ElToritoBootImage *bootimg;
@@ -260,6 +314,7 @@ int Xorriso_set_system_area(struct XorrisO *xorriso, struct burn_drive *drive,
  uint32_t offst;
  enum burn_disc_status state;
 
+ Xorriso_alloc_meM(buf, char, 32768);
  system_area_options= xorriso->system_area_options;
  memset(buf, 0, 32768);
  if(xorriso->system_area_disk_path[0] == 0) {
@@ -379,6 +434,7 @@ do_set:;
 ex:;
  if(fp != NULL && fp != stdin)
    fclose(fp);
+ Xorriso_free_meM(buf);
  return(ret);
 } 
 
@@ -390,7 +446,7 @@ ex:;
 int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
 {
  int ret;
- char sfe[5*SfileadrL], *cpt;
+ char *cpt;
  struct burn_drive_info *source_dinfo;
  struct burn_drive *source_drive;
  IsoImage *image= NULL;
@@ -447,8 +503,8 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
    load_size= ((stbuf.st_size / (off_t) 512) +
               !!(stbuf.st_size % (off_t) 512)) * 512;
  }
- sprintf(xorriso->info_text, "Adding boot image %s",
-         Text_shellsafe(bin_path, sfe, 0));
+ sprintf(xorriso->info_text, "Adding boot image ");
+ Text_shellsafe(bin_path, xorriso->info_text, 1);
  Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
 
  if(emul == 0)
@@ -461,8 +517,8 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
  ret= Xorriso_node_from_path(xorriso, image, bin_path, &node, 1);
  if(ret <= 0) {
    sprintf(xorriso->info_text,
-           "Cannot find in ISO image: -boot_image ... bin_path=%s",
-           Text_shellsafe(bin_path, sfe, 0));
+           "Cannot find in ISO image: -boot_image ... bin_path=");
+   Text_shellsafe(bin_path, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    {ret= 0; goto ex;}
  }
@@ -482,8 +538,8 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
    if(ret > 0) {
      if(!xorriso->do_overwrite) {
        sprintf(xorriso->info_text,
-               "May not overwite existing -boot_image ... cat_path=%s",
-               Text_shellsafe(xorriso->boot_image_cat_path, sfe, 0));
+               "May not overwite existing -boot_image ... cat_path=");
+       Text_shellsafe(xorriso->boot_image_cat_path, xorriso->info_text, 1);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        {ret= 0; goto ex;}
      }
@@ -491,8 +547,8 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
                       8 | (xorriso->do_overwrite == 1));
      if(ret != 1) {
        sprintf(xorriso->info_text,
-               "Could not remove existing -boot_image cat_path=%s",
-               Text_shellsafe(xorriso->boot_image_cat_path, sfe, 0));
+               "Could not remove existing -boot_image cat_path=");
+       Text_shellsafe(xorriso->boot_image_cat_path, xorriso->info_text, 1);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        {ret= 0; goto ex;}
      }
@@ -554,8 +610,8 @@ ex:;
 int Xorriso_write_session(struct XorrisO *xorriso, int flag)
 {
  int ret, relax= 0, i, pacifier_speed= 0, data_lba, ext, is_bootable= 0;
- int freshly_bootable= 0, hide_attr, pad_by_libisofs= 0;
- char xorriso_id[256], *img_id, sfe[5*SfileadrL], *out_cs, *part_image;
+ int freshly_bootable= 0, hide_attr, pad_by_libisofs= 0, signal_mode;
+ char *xorriso_id= NULL, *img_id, *sfe= NULL, *out_cs, *part_image;
  struct isoburn_imgen_opts *sopts= NULL;
  struct burn_drive_info *dinfo, *source_dinfo;
  struct burn_drive *drive, *source_drive;
@@ -569,13 +625,17 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  IsoImage *image= NULL;
  IsoNode *root_node;
  int profile_number;
- char profile_name[80];
+ char *profile_name= NULL;
  IsoBoot *bootcat_node;
  uint32_t padding;
 
+ Xorriso_alloc_meM(sfe, char, 5 * SfileadrL);
+ Xorriso_alloc_meM(xorriso_id, char, 256);
+ Xorriso_alloc_meM(profile_name, char, 80);
+
  ret= Xorriso_finish_hl_update(xorriso, 0);
  if(ret <= 0)
-   return(ret);
+   goto ex;
 
  out_cs= xorriso->out_charset;
  if(out_cs == NULL)
@@ -584,12 +644,12 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  if(!(flag & 1)) {
    ret= Xorriso_auto_format(xorriso, 0);
    if(ret <=0 )
-     return(0);
+     {ret= 0; goto ex;}
  }
  ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
                                 "on attempt to write", 2);
  if(ret<=0)
-   return(0);
+   goto ex;
 
  s= isoburn_disc_get_status(drive);
  if(xorriso->out_drive_handle == xorriso->in_drive_handle) {
@@ -628,6 +688,10 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  else if(ret == 3)
    pacifier_speed= 2;
 
+ ret= Xorriso_check_multi(xorriso, drive, 0);
+ if(ret<=0) 
+   goto ex;
+
  ret= isoburn_igopt_new(&sopts, 0);
  if(ret<=0) {
    Xorriso_process_msg_queues(xorriso, 0);
@@ -827,7 +891,7 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
    /* Padding to be done by libisofs, not by libburn.
    */
    padding= xorriso->padding / 2048;
-   if(xorriso->padding > padding * 2048)
+   if((uint32_t) xorriso->padding > padding * 2048)
      padding++;
 /*
 fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
@@ -926,7 +990,12 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
    goto cancel_iso;
 
  /* Important: do not return until burn_is_aborting() was checked */
- Xorriso_set_signal_handling(xorriso, 1);
+
+ signal_mode= 1;
+ ret= burn_drive_get_drive_role(drive);
+ if(ret == 1)
+   signal_mode|= 2;
+ Xorriso_set_signal_handling(xorriso, signal_mode);
 
  /* De-activate eventual target file truncation in dummy mode */
  ret= isoburn_set_truncate(drive, (!xorriso->do_dummy) | 2 | 4);
@@ -1002,6 +1071,9 @@ ex:;
  isoburn_igopt_destroy(&sopts, 0);
  Xorriso_process_msg_queues(xorriso,0);
  Xorriso_append_scdbackup_record(xorriso, 0);
+ Xorriso_free_meM(sfe);
+ Xorriso_free_meM(xorriso_id);
+ Xorriso_free_meM(profile_name);
  return(ret);
 
 cancel_iso:;
@@ -1253,7 +1325,7 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
 */
 int Xorriso_blank_media(struct XorrisO *xorriso, int flag)
 {
- int ret, do_deformat= 0;
+ int ret, do_deformat= 0, signal_mode;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
  enum burn_disc_status disc_state;
@@ -1290,10 +1362,14 @@ int Xorriso_blank_media(struct XorrisO *xorriso, int flag)
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "HINT", 0);
      flag&= ~1;
    }
- } else if(disc_state == BURN_DISC_BLANK) {
-   sprintf(xorriso->info_text,"Blank media detected. Will leave it untouched");
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
-   return 2;
+ }
+ if(disc_state == BURN_DISC_BLANK) {
+   if(!do_deformat) {
+     sprintf(xorriso->info_text,
+             "Blank media detected. Will leave it untouched");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
+     return 2;
+   }
  } else if(disc_state==BURN_DISC_FULL || disc_state==BURN_DISC_APPENDABLE) {
    ;
  } else if(disc_state == BURN_DISC_EMPTY) {
@@ -1322,7 +1398,11 @@ int Xorriso_blank_media(struct XorrisO *xorriso, int flag)
  Xorriso_info(xorriso,0);
 
  /* Important: do not return until burn_is_aborting() was checked */
- Xorriso_set_signal_handling(xorriso, 1);
+ signal_mode= 1;
+ ret= burn_drive_get_drive_role(drive);
+ if(ret == 1)
+   signal_mode|= 2;
+ Xorriso_set_signal_handling(xorriso, signal_mode);
 
  if(do_deformat)
    burn_disc_erase(drive, (flag&1));
@@ -1370,7 +1450,7 @@ int Xorriso_blank_media(struct XorrisO *xorriso, int flag)
 */
 int Xorriso_format_media(struct XorrisO *xorriso, off_t in_size, int flag)
 {
- int ret, mode_flag= 0, index, status, num_formats;
+ int ret, mode_flag= 0, index, status, num_formats, signal_mode;
  unsigned dummy;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
@@ -1481,7 +1561,11 @@ int Xorriso_format_media(struct XorrisO *xorriso, off_t in_size, int flag)
    size= in_size;
 
  /* Important: do not return until burn_is_aborting() was checked */
- Xorriso_set_signal_handling(xorriso, 1);
+ signal_mode= 1;
+ ret= burn_drive_get_drive_role(drive);
+ if(ret == 1)
+   signal_mode|= 2;
+ Xorriso_set_signal_handling(xorriso, signal_mode);
 
  burn_disc_format(drive, size, mode_flag);
 
@@ -1612,33 +1696,40 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
                        char *track_source, off_t tsize, int flag)
 {
  int ret, fd, unpredicted_size, profile_number, is_cd= 0, dummy, nwa= -1;
- int isosize= -1, do_isosize, is_bd= 0;
-
+ int isosize= -1, do_isosize, is_bd= 0, signal_mode;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
- struct burn_write_opts *burn_options;
+ struct burn_write_opts *burn_options= NULL;
  struct burn_disc *disc= NULL;
- struct burn_session *session;
- struct burn_track *track;
+ struct burn_session *session= NULL;
+ struct burn_track *track= NULL;
  struct stat stbuf; 
  off_t fixed_size= 0;
  struct burn_source *data_src= NULL, *fifo_src= NULL;
  enum burn_disc_status disc_state;
- char reasons[BURN_REASONS_LEN], sfe[5*SfileadrL], profile_name[80];
- char head_buffer[64*1024];
+ char *reasons= NULL, *profile_name= NULL;
+ char *head_buffer= NULL;
+
+ Xorriso_alloc_meM(reasons, char, BURN_REASONS_LEN);
+ Xorriso_alloc_meM(profile_name, char, 80);
+ Xorriso_alloc_meM(head_buffer, char, 64 * 1024);
 
  ret= Xorriso_may_burn(xorriso, 0);
  if(ret <= 0)
-   return(0);
+   {ret= 0; goto ex;}
  ret= Xorriso_auto_format(xorriso, 0);
  if(ret <=0 )
-   return(0);
+   {ret= 0; goto ex;}
 
  do_isosize= !!(flag&2);
  ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
                                 "on attempt to burn track", 2);
  if(ret<=0)
-   return(0);
+   {ret= 0; goto ex;}
+
+ ret= Xorriso_check_multi(xorriso, drive, 1);
+ if(ret<=0) 
+   goto ex;
  ret= Xorriso_make_write_options(xorriso, drive, &burn_options, 0);
  if(ret<=0)
    goto ex;
@@ -1670,8 +1761,8 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
  if(fd>=0)
    data_src= burn_fd_source_new(fd, -1, fixed_size);
  if(data_src==NULL) {
-   sprintf(xorriso->info_text, "Could not open data source %s",
-           Text_shellsafe(track_source,sfe,0));
+   sprintf(xorriso->info_text, "Could not open data source ");
+   Text_shellsafe(track_source, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE", 0);
    ret= 0; goto ex;
  }
@@ -1812,7 +1903,11 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
    goto ex;
 
  /* Important: do not return until burn_is_aborting() was checked */
- Xorriso_set_signal_handling(xorriso, 1);
+ signal_mode= 1;
+ ret= burn_drive_get_drive_role(drive);
+ if(ret == 1)
+   signal_mode|= 2;
+ Xorriso_set_signal_handling(xorriso, signal_mode);
 
  xorriso->run_state= 1; /* Indicate that burning has started */
  burn_disc_write(burn_options, disc);
@@ -1837,18 +1932,28 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
    if(ret <= 0)
      goto ex;
  }
- sprintf(xorriso->info_text, "Writing to %s completed successfully.\n\n",
-         Text_shellsafe(xorriso->outdev,sfe,0));
+ sprintf(xorriso->info_text, "Writing to ");
+ Text_shellsafe(xorriso->outdev, xorriso->info_text, 1);
+ strcat(xorriso->info_text, " completed successfully.\n\n");
  Xorriso_info(xorriso, 0);
  ret= 1;
 ex:;
  Xorriso_process_msg_queues(xorriso,0);
  if(disc!=NULL)
    burn_disc_free(disc);
+ if(session != NULL)
+   burn_session_free(session);
+ if(track != NULL)
+   burn_track_free(track);
+ if(burn_options != NULL)
+   burn_write_opts_free(burn_options);
  if(xorriso->pacifier_fifo!=NULL)
    burn_source_free(xorriso->pacifier_fifo);
  xorriso->pacifier_fifo= NULL;
  xorriso->run_state= 0; /* Indicate that burning has ended */
+ Xorriso_free_meM(reasons);
+ Xorriso_free_meM(profile_name);
+ Xorriso_free_meM(head_buffer);
  return(ret);
 }
 
@@ -1859,7 +1964,7 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
  char *npt, *cpt;
  int l, was, value, ret;
  struct isoburn_imgen_opts *opts= NULL;
- char msg[160];
+ char *msg= NULL;
  off_t limit;
 
  was= xorriso->relax_compliance;
@@ -2028,10 +2133,13 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
        if(xorriso->file_size_limit < Xorriso_default_file_size_limiT)
          xorriso->file_size_limit= Xorriso_default_file_size_limiT;
      } else {
+       Xorriso_alloc_meM(msg, char, 160); 
        sprintf(msg,
              "-compliance iso_9660_level=%d : Only 1, 2, or 3 are permissible",
              value);
        Xorriso_msgs_submit(xorriso, 0, msg, 0, "FAILURE", 0);
+       Xorriso_free_meM(msg);
+       msg= NULL;
        xorriso->relax_compliance= was;
        return(0);
      }
@@ -2066,6 +2174,9 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
    }
  }
  return(1);
+ex:;
+ Xorriso_free_meM(msg);
+ return(ret);
 }
 
 
@@ -2370,15 +2481,16 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
 int Xorriso_set_system_area_path(struct XorrisO *xorriso, char *path, int flag)
 {
  int ret;
- char eff_src[SfileadrL];
+ char *eff_src= NULL;
 
  if(path[0] == 0) {
    xorriso->system_area_disk_path[0]= 0;
-   return(1);
+   {ret= 1; goto ex;}
  }
+ Xorriso_alloc_meM(eff_src, char, SfileadrL);
  ret= Xorriso_normalize_img_path(xorriso, xorriso->wdx, path, eff_src, 2|4|16);
  if(ret < 0)
-   return(ret);
+   goto ex;
  if(ret == 0) {
    sprintf(xorriso->info_text,
            "Given path does not exist on disk: -boot_image system_area=");
@@ -2390,11 +2502,51 @@ int Xorriso_set_system_area_path(struct XorrisO *xorriso, char *path, int flag)
            "Given path leads to a directory: -boot_image system_area=");
    Text_shellsafe(eff_src, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(0);
+   {ret= 0; goto ex;}
  }
  ret= Sfile_str(xorriso->system_area_disk_path, eff_src, 0);
  if(ret <= 0)
-   return(-1);
- return(1);
+   {ret= -1; goto ex;}
+ ret= 1;
+ex:
+ Xorriso_free_meM(eff_src);
+ return(ret);
+}
+
+
+/* @param flag bit0=force burn_disc_close_damaged()
+*/
+int Xorriso_close_damaged(struct XorrisO *xorriso, int flag)
+{
+ int ret;
+ struct burn_drive_info *dinfo;
+ struct burn_drive *drive;
+ struct burn_write_opts *burn_options= NULL;
+
+ if(xorriso->volset_change_pending) {
+   sprintf(xorriso->info_text,
+           "Image changes pending. -commit or -rollback first");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   ret= 0; goto ex;
+ }
+ ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
+                                "on attempt to closed damaged session", 2);
+ if(ret<=0)
+   goto ex;
+ ret= Xorriso_make_write_options(xorriso, drive, &burn_options, 0);
+ if(ret <= 0)
+   goto ex;
+ ret= burn_disc_close_damaged(burn_options, flag & 1);
+ Xorriso_process_msg_queues(xorriso, 0);
+ Xorriso_option_dev(xorriso, "", 3 | 4); /* Give up drives */
+ if(ret <= 0)
+   goto ex;
+
+ ret= 1;
+ex:;
+ Xorriso_process_msg_queues(xorriso, 0);
+ if(burn_options != NULL)
+   burn_write_opts_free(burn_options);
+ return(ret); 
 }
 
