@@ -18,8 +18,8 @@
 /* The minimum version of libisoburn to be used with this version of xorriso
 */
 #define xorriso_libisoburn_req_major  1
-#define xorriso_libisoburn_req_minor  0
-#define xorriso_libisoburn_req_micro  8
+#define xorriso_libisoburn_req_minor  1
+#define xorriso_libisoburn_req_micro  0
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -570,5 +570,13 @@ int Xorriso_clone_under(struct XorrisO *xorriso, char *origin, char *dest,
 int Xorriso_mark_update_merge(struct XorrisO *xorriso, char *path,
                               void *node, int flag);
 
+/* @param flag bit0= asynchronous handling (else catch thread, wait, and exit)
+*/
+int Xorriso_set_signal_handling(struct XorrisO *xorriso, int flag);
+
+/* @param flag bit0=force burn_disc_close_damaged()
+*/
+int Xorriso_close_damaged(struct XorrisO *xorriso, int flag);
+
 #endif /* Xorrisoburn_includeD */
 
